@@ -1,4 +1,5 @@
 import pytest
+from azure.core.exceptions import ResourceNotFoundError
 from azfs.clients.blob_client import AzBlobClient
 from azfs.clients.datalake_client import AzDataLakeClient
 from azfs.clients.client_interface import ClientInterface
@@ -10,7 +11,8 @@ class TestClientInterface:
     def test_not_implemented_error(self):
         client_interface = ClientInterface(credential="")
         # the file below is not exists
-        path = "https://testazfs.blob.core.windows.net/test_caontainer/test.csv"
+        account_url = "https://testazfs.blob.core.windows.net/"
+        path = f"{account_url}test_caontainer/test.csv"
         file_path = "test_caontainer"
 
         with pytest.raises(NotImplementedError):
@@ -34,6 +36,9 @@ class TestClientInterface:
         with pytest.raises(NotImplementedError):
             client_interface.get_file_client_from_path(path=path)
 
+        with pytest.raises(NotImplementedError):
+            client_interface.get_service_client_from_url(account_url=account_url)
+
 
 class TestReadCsv:
 
@@ -51,6 +56,20 @@ class TestReadCsv:
         assert "age" in columns
         assert len(df.index) == 2
 
+    def test_blob_read_csv_gz(self, mocker, _get_csv_gz, var_azc):
+        mocker.patch.object(AzBlobClient, "_get", _get_csv_gz)
+
+        # the file below is not exists
+        path = "https://testazfs.blob.core.windows.net/test_caontainer/test.csv.gz"
+
+        # read data from not-exist path
+        with var_azc:
+            df = pd.read_csv_az(path)
+        columns = df.columns
+        assert "name" in columns
+        assert "age" in columns
+        assert len(df.index) == 2
+
     def test_dfs_read_csv(self, mocker, _get_csv, var_azc):
         mocker.patch.object(AzDataLakeClient, "_get", _get_csv)
 
@@ -250,8 +269,8 @@ class TestRm:
 
 
 class TestExists:
-    def test_blob_exists(self, mocker, _ls, var_azc):
-        mocker.patch.object(AzBlobClient, "_ls", _ls)
+    def test_blob_exists(self, mocker, _get_csv, var_azc):
+        mocker.patch.object(AzBlobClient, "_get", _get_csv)
 
         # the file below is not exists
         path = "https://testazfs.blob.core.windows.net/test_caontainer/test1.csv"
@@ -259,13 +278,16 @@ class TestExists:
         result = var_azc.exists(path=path)
         assert result
 
+        # set to raise exception
+        _get_csv.side_effect = ResourceNotFoundError
+        mocker.patch.object(AzBlobClient, "_get", _get_csv)
         # the file below is not exists
         path = "https://testazfs.blob.core.windows.net/test_caontainer/test3.csv"
         result = var_azc.exists(path=path)
         assert not result
 
-    def test_dfs_exists(self, mocker, _ls, var_azc):
-        mocker.patch.object(AzDataLakeClient, "_ls", _ls)
+    def test_dfs_exists(self, mocker, _get_csv, var_azc):
+        mocker.patch.object(AzDataLakeClient, "_get", _get_csv)
 
         # the file below is not exists
         path = "https://testazfs.dfs.core.windows.net/test_caontainer/test1.csv"
@@ -273,6 +295,9 @@ class TestExists:
         result = var_azc.exists(path=path)
         assert result
 
+        # set to raise exception
+        _get_csv.side_effect = ResourceNotFoundError
+        mocker.patch.object(AzDataLakeClient, "_get", _get_csv)
         # the file below is not exists
         path = "https://testazfs.dfs.core.windows.net/test_caontainer/test3.csv"
         result = var_azc.exists(path=path)
